/* Automatically generated by po2tbl.sed from sh-utils.pot.  */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include "libgettext.h"

const struct _msg_ent _msg_tbl[] = {
  {"", 1},
  {"invalid argument %s for `%s'", 2},
  {"ambiguous argument %s for `%s'", 3},
  {"Valid arguments are:", 4},
  {"write error", 5},
  {"Unknown system error", 6},
  {"%s: option `%s' is ambiguous\n", 7},
  {"%s: option `--%s' doesn't allow an argument\n", 8},
  {"%s: option `%c%s' doesn't allow an argument\n", 9},
  {"%s: option `%s' requires an argument\n", 10},
  {"%s: unrecognized option `--%s'\n", 11},
  {"%s: unrecognized option `%c%s'\n", 12},
  {"%s: illegal option -- %c\n", 13},
  {"%s: invalid option -- %c\n", 14},
  {"%s: option requires an argument -- %c\n", 15},
  {"%s: option `-W %s' is ambiguous\n", 16},
  {"%s: option `-W %s' doesn't allow an argument\n", 17},
  {"block size", 18},
  {"cannot create directory %s", 19},
  {"%s exists but is not a directory", 20},
  {"cannot change owner and/or group of %s", 21},
  {"cannot chdir to directory, %s", 22},
  {"cannot change permissions of %s", 23},
  {"memory exhausted", 24},
  {"`", 25},
  {"'", 26},
  {"^[yY]", 27},
  {"^[nN]", 28},
  {"cannot output U+%04X: iconv function not usable", 29},
  {"U+%04X: character out of range", 30},
  {"cannot convert U+%04X to local character set", 31},
  {"cannot output U+%04X: iconv function not available", 32},
  {"invalid user", 33},
  {"invalid group", 34},
  {"cannot get the login group of a numeric UID", 35},
  {"cannot omit both user and group", 36},
  {"Copyright (C) 2000 Free Software Foundation, Inc.", 37},
  {"Written by %s.\n", 38},
  {"\
This is free software; see the source for copying conditions.  There is NO\n\
warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n", 39},
  {"Try `%s --help' for more information.\n", 40},
  {"\
Usage: %s NAME [SUFFIX]\n\
  or:  %s OPTION\n", 41},
  {"\
Print NAME with any leading directory components removed.\n\
If specified, also remove a trailing SUFFIX.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n", 42},
  {"\
\n\
Report bugs to <bug-sh-utils@gnu.org>.", 43},
  {"too few arguments", 44},
  {"too many arguments", 45},
  {"\
Usage: %s NEWROOT [COMMAND...]\n\
  or:  %s OPTION\n", 46},
  {"\
Run COMMAND with root directory set to NEWROOT.\n\
\n\
      --help       display this help and exit\n\
      --version    output version information and exit\n\
\n\
If no command is given, run ``${SHELL} -i'' (default: /bin/sh).\n", 47},
  {"cannot change root directory to %s", 48},
  {"cannot chdir to root directory", 49},
  {"cannot execute %s", 50},
  {"\
Usage: %s [OPTION]... [+FORMAT]\n\
  or:  %s [-u|--utc|--universal] [MMDDhhmm[[CC]YY][.ss]]\n", 51},
  {"\
Display the current time in the given FORMAT, or set the system date.\n\
\n\
  -d, --date=STRING         display time described by STRING, not `now'\n\
  -f, --file=DATEFILE       like --date once for each line of DATEFILE\n\
  -I, --iso-8601[=TIMESPEC] output an ISO-8601 compliant date/time string.\n\
                            TIMESPEC=`date' (or missing) for date only,\n\
                            `hours', `minutes', or `seconds' for date and\n\
                            time to the indicated precision.\n\
  -r, --reference=FILE      display the last modification time of FILE\n\
  -R, --rfc-822             output RFC-822 compliant date string\n\
  -s, --set=STRING          set time described by STRING\n\
  -u, --utc, --universal    print or set Coordinated Universal Time\n\
      --help                display this help and exit\n\
      --version             output version information and exit\n", 52},
  {"\
\n\
FORMAT controls the output.  The only valid option for the second form\n\
specifies Coordinated Universal Time.  Interpreted sequences are:\n\
\n\
  %%%%   a literal %%\n\
  %%a   locale's abbreviated weekday name (Sun..Sat)\n\
  %%A   locale's full weekday name, variable length (Sunday..Saturday)\n\
  %%b   locale's abbreviated month name (Jan..Dec)\n\
  %%B   locale's full month name, variable length (January..December)\n\
  %%c   locale's date and time (Sat Nov 04 12:02:33 EST 1989)\n\
  %%d   day of month (01..31)\n\
  %%D   date (mm/dd/yy)\n\
  %%e   day of month, blank padded ( 1..31)\n\
  %%h   same as %%b\n\
  %%H   hour (00..23)\n\
  %%I   hour (01..12)\n\
  %%j   day of year (001..366)\n\
  %%k   hour ( 0..23)\n\
  %%l   hour ( 1..12)\n\
  %%m   month (01..12)\n\
  %%M   minute (00..59)\n\
  %%n   a newline\n\
  %%p   locale's AM or PM\n\
  %%r   time, 12-hour (hh:mm:ss [AP]M)\n\
  %%s   seconds since `00:00:00 1970-01-01 UTC' (a GNU extension)\n\
  %%S   second (00..60)\n\
  %%t   a horizontal tab\n\
  %%T   time, 24-hour (hh:mm:ss)\n\
  %%U   week number of year with Sunday as first day of week (00..53)\n\
  %%V   week number of year with Monday as first day of week (01..53)\n\
  %%w   day of week (0..6);  0 represents Sunday\n\
  %%W   week number of year with Monday as first day of week (00..53)\n\
  %%x   locale's date representation (mm/dd/yy)\n\
  %%X   locale's time representation (%%H:%%M:%%S)\n\
  %%y   last two digits of year (00..99)\n\
  %%Y   year (1970...)\n\
  %%z   RFC-822 style numeric timezone (-0500) (a nonstandard extension)\n\
  %%Z   time zone (e.g., EDT), or nothing if no time zone is determinable\n\
\n\
By default, date pads numeric fields with zeroes.  GNU date recognizes\n\
the following modifiers between `%%' and a numeric directive.\n\
\n\
  `-' (hyphen) do not pad the field\n\
  `_' (underscore) pad the field with spaces\n", 53},
  {"standard input", 54},
  {"invalid date `%s'", 55},
  {"the options to specify dates for printing are mutually exclusive", 56},
  {"the options to print and set the time may not be used together", 57},
  {"too many non-option arguments", 58},
  {"\
the argument `%s' lacks a leading `+';\n\
When using an option to specify date(s), any non-option\n\
argument must be a format string beginning with `+'.", 59},
  {"undefined", 60},
  {"cannot set date", 61},
  {"\
Usage: %s NAME\n\
  or:  %s OPTION\n", 62},
  {"\
Print NAME with its trailing /component removed; if NAME contains no /'s,\n\
output `.' (meaning the current directory).\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n", 63},
  {"Usage: %s [OPTION]... [STRING]...\n", 64},
  {"\
Echo the STRING(s) to standard output.\n\
\n\
  -n              do not output the trailing newline\n\
  -e              enable interpretation of the backslash-escaped characters\n\
                    listed below\n\
  -E              disable interpretation of those sequences in STRINGs\n\
      --help      display this help and exit (should be alone)\n\
      --version   output version information and exit (should be alone)\n\
\n\
Without -E, the following sequences are recognized and interpolated:\n\
\n\
  \\NNN   the character whose ASCII code is NNN (octal)\n\
  \\\\     backslash\n\
  \\a     alert (BEL)\n\
  \\b     backspace\n\
  \\c     suppress trailing newline\n\
  \\f     form feed\n\
  \\n     new line\n\
  \\r     carriage return\n\
  \\t     horizontal tab\n\
  \\v     vertical tab\n", 65},
  {"Usage: %s [OPTION]... [-] [NAME=VALUE]... [COMMAND [ARG]...]\n", 66},
  {"\
Set each NAME to VALUE in the environment and run COMMAND.\n\
\n\
  -i, --ignore-environment   start with an empty environment\n\
  -u, --unset=NAME           remove variable from the environment\n\
      --help                 display this help and exit\n\
      --version              output version information and exit\n\
\n\
A mere - implies -i.  If no COMMAND, print the resulting environment.\n", 67},
  {"\
Usage: %s EXPRESSION\n\
  or:  %s OPTION\n", 68},
  {"\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n\
\n", 69},
  {"\
Print the value of EXPRESSION to standard output.  A blank line below\n\
separates increasing precedence groups.  EXPRESSION may be:\n\
\n\
  ARG1 | ARG2       ARG1 if it is neither null nor 0, otherwise ARG2\n\
\n\
  ARG1 & ARG2       ARG1 if neither argument is null or 0, otherwise 0\n\
\n\
  ARG1 < ARG2       ARG1 is less than ARG2\n\
  ARG1 <= ARG2      ARG1 is less than or equal to ARG2\n\
  ARG1 = ARG2       ARG1 is equal to ARG2\n\
  ARG1 != ARG2      ARG1 is unequal to ARG2\n\
  ARG1 >= ARG2      ARG1 is greater than or equal to ARG2\n\
  ARG1 > ARG2       ARG1 is greater than ARG2\n\
\n\
  ARG1 + ARG2       arithmetic sum of ARG1 and ARG2\n\
  ARG1 - ARG2       arithmetic difference of ARG1 and ARG2\n\
\n\
  ARG1 * ARG2       arithmetic product of ARG1 and ARG2\n\
  ARG1 / ARG2       arithmetic quotient of ARG1 divided by ARG2\n\
  ARG1 %% ARG2       arithmetic remainder of ARG1 divided by ARG2\n\
\n\
  STRING : REGEXP   anchored pattern match of REGEXP in STRING\n\
\n\
  match STRING REGEXP        same as STRING : REGEXP\n\
  substr STRING POS LENGTH   substring of STRING, POS counted from 1\n\
  index STRING CHARS         index in STRING where any CHARS is found, or 0\n\
  length STRING              length of STRING\n\
  quote TOKEN                interpret TOKEN as a string, even if it is a\n\
                               keyword like `match' or an operator like `/'\n\
\n\
  ( EXPRESSION )             value of EXPRESSION\n", 70},
  {"\
\n\
Beware that many operators need to be escaped or quoted for shells.\n\
Comparisons are arithmetic if both ARGs are numbers, else lexicographical.\n\
Pattern matches return the string matched between \\( and \\) or null; if\n\
\\( and \\) are not used, they return the number of characters matched or \
0.\n", 71},
  {"syntax error", 72},
  {"\
warning: unportable BRE: `%s': using `^' as the first character\n\
of the basic regular expression is not portable; it is being ignored", 73},
  {"\
Usage: %s [NUMBER]...\n\
  or:  %s OPTION\n", 74},
  {"\
Print factors of each NUMBER; read standard input with no arguments.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n\
\n\
  Print the prime factors of all specified integer NUMBERs.  If no \
arguments\n\
  are specified on the command line, they are read from standard input.\n", 75},
  {"`%s' is not a valid positive integer", 76},
  {"\
Usage: %s [ignored command line arguments]\n\
  or:  %s OPTION\n\
Exit with a status code indicating failure.\n\
\n\
These option names may not be abbreviated.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n", 77},
  {"\
Usage: %s\n\
  or:  %s OPTION\n\
Print the numeric identifier (in hexadecimal) for the current host.\n\
\n\
      --help       display this help and exit\n\
      --version    output version information and exit\n\
\n", 78},
  {"\
Usage: %s [NAME]\n\
  or:  %s OPTION\n\
Print or set the hostname of the current system.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n", 79},
  {"cannot set hostname to `%s'", 80},
  {"cannot set hostname; this system lacks the functionality", 81},
  {"cannot determine hostname", 82},
  {"Usage: %s [OPTION]... [USERNAME]\n", 83},
  {"\
Print information for USERNAME, or the current user.\n\
\n\
  -a              ignore, for compatibility with other versions\n\
  -g, --group     print only the group ID\n\
  -G, --groups    print only the supplementary groups\n\
  -n, --name      print a name instead of a number, for -ugG\n\
  -r, --real      print the real ID instead of effective ID, for -ugG\n\
  -u, --user      print only the user ID\n\
      --help      display this help and exit\n\
      --version   output version information and exit\n\
\n\
Without any OPTION, print some useful set of identified information.\n", 84},
  {"cannot print only user and only group", 85},
  {"cannot print only names or real IDs in default format", 86},
  {"%s: No such user", 87},
  {"cannot find name for user ID %u", 88},
  {"cannot find name for group ID %u", 89},
  {"cannot get supplemental group list", 90},
  {" groups=", 91},
  {"Usage: %s [OPTION]\n", 92},
  {"\
Print the name of the current user.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n", 93},
  {"%s: no login name\n", 94},
  {"Usage: %s [OPTION] [COMMAND [ARG]...]\n", 95},
  {"\
Run COMMAND with an adjusted scheduling priority.\n\
With no COMMAND, print the current scheduling priority.  ADJUST is 10\n\
by default.  Range goes from -20 (highest priority) to 19 (lowest).\n\
\n\
  -ADJUST                   increment priority by ADJUST first\n\
  -n, --adjustment=ADJUST   same as -ADJUST\n\
      --help                display this help and exit\n\
      --version             output version information and exit\n", 96},
  {"invalid option `%s'", 97},
  {"invalid priority `%s'", 98},
  {"a command must be given with an adjustment", 99},
  {"cannot get priority", 100},
  {"cannot set priority", 101},
  {"Usage: %s [OPTION]... NAME...\n", 102},
  {"\
Diagnose unportable constructs in NAME.\n\
\n\
  -p, --portability   check for all POSIX systems, not only this one\n\
      --help          display this help and exit\n\
      --version       output version information and exit\n", 103},
  {"path `%s' contains nonportable character `%c'", 104},
  {"`%s' is not a directory", 105},
  {"directory `%s' is not searchable", 106},
  {"name `%s' has length %d; exceeds limit of %d", 107},
  {"path `%s' has length %d; exceeds limit of %d", 108},
  {"Login name: ", 109},
  {"In real life: ", 110},
  {"???\n", 111},
  {"Directory: ", 112},
  {"Shell: ", 113},
  {"Project: ", 114},
  {"Plan:\n", 115},
  {"Login", 116},
  {"   Name", 117},
  {"TTY", 118},
  {"Idle", 119},
  {"When", 120},
  {"Where", 121},
  {"Usage: %s [OPTION]... [USER]...\n", 122},
  {"\
\n\
  -l              do long format output\n\
  -b              omit the user's home directory and shell in long format\n\
  -h              omit the user's project file in long format\n\
  -p              omit the user's plan file in long format\n\
  -s              do short format output, this is the default\n\
  -f              omit the line of column headings in short format\n\
  -w              omit the user's full name in short format\n\
  -i              omit the user's full name and remote host in short format\n\
  -q              omit the user's full name, remote host and idle time\n\
                  in short format\n\
      --help      display this help and exit\n\
      --version   output version information and exit\n\
\n\
A lightweight `finger' program;  print user information.\n\
The utmp file will be %s.\n", 123},
  {"\
Usage: %s [VARIABLE]...\n\
  or:  %s OPTION\n\
If no environment VARIABLE specified, print them all.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n", 124},
  {"\
warning: %s: character(s) following character constant have been ignored", 125},
  {"\
Usage: %s FORMAT [ARGUMENT]...\n\
  or:  %s OPTION\n", 126},
  {"\
Print ARGUMENT(s) according to FORMAT.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n\
\n\
FORMAT controls the output as in C printf.  Interpreted sequences are:\n\
\n\
  \\\"      double quote\n\
  \\0NNN   character with octal value NNN (0 to 3 digits)\n\
  \\\\      backslash\n\
  \\a      alert (BEL)\n\
  \\b      backspace\n\
  \\c      produce no further output\n\
  \\f      form feed\n\
  \\n      new line\n\
  \\r      carriage return\n\
  \\t      horizontal tab\n\
  \\v      vertical tab\n\
  \\xNNN   byte with hexadecimal value NNN (1 to 3 digits)\n\
\n\
  \\uNNNN  character with hexadecimal value NNNN (4 digits)\n\
  \\UNNNNNNNN  character with hexadecimal value NNNNNNNN (8 digits)\n\
  %%%%      a single %%\n\
  %%b      ARGUMENT as a string with `\\' escapes interpreted\n\
\n\
and all C format specifications ending with one of diouxXfeEgGcs, with\n\
ARGUMENTs converted to proper type first.  Variable widths are handled.\n", 127},
  {"%s: expected a numeric value", 128},
  {"%s: value not completely converted", 129},
  {"missing hexadecimal number in escape", 130},
  {"invalid universal character name \\%c%0*x", 131},
  {"\\%c: invalid escape", 132},
  {"%%%c: invalid directive", 133},
  {"Usage: %s format [argument...]\n", 134},
  {"warning: excess arguments have been ignored", 135},
  {"\
Print the full filename of the current working directory.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n", 136},
  {"ignoring non-option arguments", 137},
  {"cannot get current directory", 138},
  {"\
Usage: %s [OPTION]... LAST\n\
  or:  %s [OPTION]... FIRST LAST\n\
  or:  %s [OPTION]... FIRST INCREMENT LAST\n", 139},
  {"\
Print numbers from FIRST to LAST, in steps of INCREMENT.\n\
\n\
  -f, --format FORMAT      use printf(3) style FORMAT (default: %%g)\n\
  -s, --separator STRING   use STRING to separate numbers (default: \\n)\n\
  -w, --equal-width        equalize width by padding with leading zeroes\n\
      --help               display this help and exit\n\
      --version            output version information and exit\n\
\n\
If FIRST or INCREMENT is omitted, it defaults to 1.\n\
FIRST, INCREMENT, and LAST are interpreted as floating point values.\n\
INCREMENT should be positive if FIRST is smaller than LAST, and negative\n\
otherwise.  When given, the FORMAT argument must contain exactly one of\n\
the printf-style, floating point output formats %%e, %%f, %%g\n", 140},
  {"invalid floating point argument: %s", 141},
  {"\
when the starting value is larger than the limit,\n\
the increment must be negative", 142},
  {"\
when the starting value is smaller than the limit,\n\
the increment must be positive", 143},
  {"invalid format string: `%s'", 144},
  {"format string may not be specified when printing equal width strings", 145},
  {"\
Usage: %s NUMBER[SUFFIX]...\n\
  or:  %s OPTION\n\
Pause for NUMBER seconds.  SUFFIX may be `s' for seconds (the default),\n\
`m' for minutes, `h' for hours or `d' for days.  Unlike most \
implementations\n\
that require NUMBER be an integer, here NUMBER may be an arbitrary floating\n\
point number.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n", 146},
  {"cannot read realtime clock", 147},
  {"invalid time interval `%s'", 148},
  {"\
Usage: %s [-F DEVICE] [--file=DEVICE] [SETTING]...\n\
  or:  %s [-F DEVICE] [--file=DEVICE] [-a|--all]\n\
  or:  %s [-F DEVICE] [--file=DEVICE] [-g|--save]\n", 149},
  {"\
Print or change terminal characteristics.\n\
\n\
  -a, --all          print all current settings in human-readable form\n\
  -g, --save         print all current settings in a stty-readable form\n\
  -F, --file=DEVICE  open and use the specified DEVICE instead of stdin\n\
      --help         display this help and exit\n\
      --version      output version information and exit\n\
\n\
Optional - before SETTING indicates negation.  An * marks non-POSIX\n\
settings.  The underlying system defines which settings are available.\n", 150},
  {"\
\n\
Special characters:\n\
* dsusp CHAR    CHAR will send a terminal stop signal once input flushed\n\
  eof CHAR      CHAR will send an end of file (terminate the input)\n\
  eol CHAR      CHAR will end the line\n\
* eol2 CHAR     alternate CHAR for ending the line\n\
  erase CHAR    CHAR will erase the last character typed\n\
  intr CHAR     CHAR will send an interrupt signal\n\
  kill CHAR     CHAR will erase the current line\n\
* lnext CHAR    CHAR will enter the next character quoted\n\
  quit CHAR     CHAR will send a quit signal\n\
* rprnt CHAR    CHAR will redraw the current line\n\
  start CHAR    CHAR will restart the output after stopping it\n\
  stop CHAR     CHAR will stop the output\n\
  susp CHAR     CHAR will send a terminal stop signal\n\
* swtch CHAR    CHAR will switch to a different shell layer\n\
* werase CHAR   CHAR will erase the last word typed\n", 151},
  {"\
\n\
Special settings:\n\
  N             set the input and output speeds to N bauds\n\
* cols N        tell the kernel that the terminal has N columns\n\
* columns N     same as cols N\n\
  ispeed N      set the input speed to N\n\
* line N        use line discipline N\n\
  min N         with -icanon, set N characters minimum for a completed read\n\
  ospeed N      set the output speed to N\n\
* rows N        tell the kernel that the terminal has N rows\n\
* size          print the number of rows and columns according to the \
kernel\n\
  speed         print the terminal speed\n\
  time N        with -icanon, set read timeout of N tenths of a second\n", 152},
  {"\
\n\
Control settings:\n\
  [-]clocal     disable modem control signals\n\
  [-]cread      allow input to be received\n\
* [-]crtscts    enable RTS/CTS handshaking\n\
  csN           set character size to N bits, N in [5..8]\n\
  [-]cstopb     use two stop bits per character (one with `-')\n\
  [-]hup        send a hangup signal when the last process closes the tty\n\
  [-]hupcl      same as [-]hup\n\
  [-]parenb     generate parity bit in output and expect parity bit in \
input\n\
  [-]parodd     set odd parity (even with `-')\n", 153},
  {"\
\n\
Input settings:\n\
  [-]brkint     breaks cause an interrupt signal\n\
  [-]icrnl      translate carriage return to newline\n\
  [-]ignbrk     ignore break characters\n\
  [-]igncr      ignore carriage return\n\
  [-]ignpar     ignore characters with parity errors\n\
* [-]imaxbel    beep and do not flush a full input buffer on a character\n\
  [-]inlcr      translate newline to carriage return\n\
  [-]inpck      enable input parity checking\n\
  [-]istrip     clear high (8th) bit of input characters\n\
* [-]iuclc      translate uppercase characters to lowercase\n\
* [-]ixany      let any character restart output, not only start character\n\
  [-]ixoff      enable sending of start/stop characters\n\
  [-]ixon       enable XON/XOFF flow control\n\
  [-]parmrk     mark parity errors (with a 255-0-character sequence)\n\
  [-]tandem     same as [-]ixoff\n", 154},
  {"\
\n\
Output settings:\n\
* bsN           backspace delay style, N in [0..1]\n\
* crN           carriage return delay style, N in [0..3]\n\
* ffN           form feed delay style, N in [0..1]\n\
* nlN           newline delay style, N in [0..1]\n\
* [-]ocrnl      translate carriage return to newline\n\
* [-]ofdel      use delete characters for fill instead of null characters\n\
* [-]ofill      use fill (padding) characters instead of timing for delays\n\
* [-]olcuc      translate lowercase characters to uppercase\n\
* [-]onlcr      translate newline to carriage return-newline\n\
* [-]onlret     newline performs a carriage return\n\
* [-]onocr      do not print carriage returns in the first column\n\
  [-]opost      postprocess output\n\
* tabN          horizontal tab delay style, N in [0..3]\n\
* tabs          same as tab0\n\
* -tabs         same as tab3\n\
* vtN           vertical tab delay style, N in [0..1]\n", 155},
  {"\
\n\
Local settings:\n\
  [-]crterase   echo erase characters as backspace-space-backspace\n\
* crtkill       kill all line by obeying the echoprt and echoe settings\n\
* -crtkill      kill all line by obeying the echoctl and echok settings\n\
* [-]ctlecho    echo control characters in hat notation (`^c')\n\
  [-]echo       echo input characters\n\
* [-]echoctl    same as [-]ctlecho\n\
  [-]echoe      same as [-]crterase\n\
  [-]echok      echo a newline after a kill character\n\
* [-]echoke     same as [-]crtkill\n\
  [-]echonl     echo newline even if not echoing other characters\n\
* [-]echoprt    echo erased characters backward, between `\\' and '/'\n\
  [-]icanon     enable erase, kill, werase, and rprnt special characters\n\
  [-]iexten     enable non-POSIX special characters\n\
  [-]isig       enable interrupt, quit, and suspend special characters\n\
  [-]noflsh     disable flushing after interrupt and quit special \
characters\n\
* [-]prterase   same as [-]echoprt\n\
* [-]tostop     stop background jobs that try to write to the terminal\n\
* [-]xcase      with icanon, escape with `\\' for uppercase characters\n", 156},
  {"\
\n\
Combination settings:\n\
* [-]LCASE      same as [-]lcase\n\
  cbreak        same as -icanon\n\
  -cbreak       same as icanon\n\
  cooked        same as brkint ignpar istrip icrnl ixon opost isig\n\
                icanon, eof and eol characters to their default values\n\
  -cooked       same as raw\n\
  crt           same as echoe echoctl echoke\n\
  dec           same as echoe echoctl echoke -ixany intr ^c erase 0177\n\
                kill ^u\n\
* [-]decctlq    same as [-]ixany\n\
  ek            erase and kill characters to their default values\n\
  evenp         same as parenb -parodd cs7\n\
  -evenp        same as -parenb cs8\n\
* [-]lcase      same as xcase iuclc olcuc\n\
  litout        same as -parenb -istrip -opost cs8\n\
  -litout       same as parenb istrip opost cs7\n\
  nl            same as -icrnl -onlcr\n\
  -nl           same as icrnl -inlcr -igncr onlcr -ocrnl -onlret\n\
  oddp          same as parenb parodd cs7\n\
  -oddp         same as -parenb cs8\n\
  [-]parity     same as [-]evenp\n\
  pass8         same as -parenb -istrip cs8\n\
  -pass8        same as parenb istrip cs7\n\
  raw           same as -ignbrk -brkint -ignpar -parmrk -inpck -istrip\n\
                -inlcr -igncr -icrnl  -ixon  -ixoff  -iuclc  -ixany\n\
                -imaxbel -opost -isig -icanon -xcase min 1 time 0\n\
  -raw          same as cooked\n\
  sane          same as cread -ignbrk brkint -inlcr -igncr icrnl\n\
                -ixoff -iuclc -ixany imaxbel opost -olcuc -ocrnl onlcr\n\
                -onocr -onlret -ofill -ofdel nl0 cr0 tab0 bs0 vt0 ff0\n\
                isig icanon iexten echo echoe echok -echonl -noflsh\n\
                -xcase -tostop -echoprt echoctl echoke, all special\n\
                characters to their default values.\n", 157},
  {"\
\n\
Handle the tty line connected to standard input.  Without arguments,\n\
prints baud rate, line discipline, and deviations from stty sane.  In\n\
settings, CHAR is taken literally, or coded as in ^c, 0x37, 0177 or\n\
127; special values ^- or undef used to disable special characters.\n", 158},
  {"only one device may be specified", 159},
  {"\
the options for verbose and stty-readable output styles are\n\
mutually exclusive", 160},
  {"when specifying an output style, modes may not be set", 161},
  {"%s: couldn't reset non-blocking mode", 162},
  {"invalid argument `%s'", 163},
  {"missing argument to `%s'", 164},
  {"%s: unable to perform all requested operations", 165},
  {"new_mode: mode\n", 166},
  {"%s: no size information for this device", 167},
  {"invalid integer argument `%s'", 168},
  {"Password:", 169},
  {"getpass: cannot open /dev/tty", 170},
  {"cannot set groups", 171},
  {"cannot set group id", 172},
  {"cannot set user id", 173},
  {"cannot run %s", 174},
  {"Usage: %s [OPTION]... [-] [USER [ARG]...]\n", 175},
  {"\
Change the effective user id and group id to that of USER.\n\
\n\
  -, -l, --login               make the shell a login shell\n\
  -c, --commmand=COMMAND       pass a single COMMAND to the shell with -c\n\
  -f, --fast                   pass -f to the shell (for csh or tcsh)\n\
  -m, --preserve-environment   do not reset environment variables\n\
  -p                           same as -m\n\
  -s, --shell=SHELL            run SHELL if /etc/shells allows it\n\
      --help                   display this help and exit\n\
      --version                output version information and exit\n\
\n\
A mere - implies -l.   If USER not given, assume root.\n", 176},
  {"user %s does not exist", 177},
  {"incorrect password", 178},
  {"using restricted shell %s", 179},
  {"warning: cannot change directory to %s", 180},
  {"Usage: %s [OPTION]... [FILE]...\n", 181},
  {"\
Copy standard input to each FILE, and also to standard output.\n\
\n\
  -a, --append              append to the given FILEs, do not overwrite\n\
  -i, --ignore-interrupts   ignore interrupt signals\n\
      --help                display this help and exit\n\
      --version             output version information and exit\n", 182},
  {"standard output", 183},
  {"read error", 184},
  {"argument expected\n", 185},
  {"integer expression expected %s\n", 186},
  {"')' expected\n", 187},
  {"')' expected, found %s\n", 188},
  {"%s: unary operator expected\n", 189},
  {"%s: binary operator expected\n", 190},
  {"before -lt", 191},
  {"after -lt", 192},
  {"before -le", 193},
  {"after -le", 194},
  {"before -gt", 195},
  {"after -gt", 196},
  {"before -ge", 197},
  {"after -ge", 198},
  {"-nt does not accept -l\n", 199},
  {"before -ne", 200},
  {"after -ne", 201},
  {"before -eq", 202},
  {"after -eq", 203},
  {"-ef does not accept -l\n", 204},
  {"unknown binary operator", 205},
  {"after -t", 206},
  {"\
Usage: %s EXPRESSION\n\
  or:  [ EXPRESSION ]\n\
  or:  %s OPTION\n", 207},
  {"\
Exit with the status determined by EXPRESSION.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n\
\n\
EXPRESSION is true or false and sets exit status.  It is one of:\n", 208},
  {"\
\n\
  ( EXPRESSION )               EXPRESSION is true\n\
  ! EXPRESSION                 EXPRESSION is false\n\
  EXPRESSION1 -a EXPRESSION2   both EXPRESSION1 and EXPRESSION2 are true\n\
  EXPRESSION1 -o EXPRESSION2   either EXPRESSION1 or EXPRESSION2 is true\n\
\n\
  [-n] STRING          the length of STRING is nonzero\n\
  -z STRING            the length of STRING is zero\n\
  STRING1 = STRING2    the strings are equal\n\
  STRING1 != STRING2   the strings are not equal\n\
\n\
  INTEGER1 -eq INTEGER2   INTEGER1 is equal to INTEGER2\n\
  INTEGER1 -ge INTEGER2   INTEGER1 is greater than or equal to INTEGER2\n\
  INTEGER1 -gt INTEGER2   INTEGER1 is greater than INTEGER2\n\
  INTEGER1 -le INTEGER2   INTEGER1 is less than or equal to INTEGER2\n\
  INTEGER1 -lt INTEGER2   INTEGER1 is less than INTEGER2\n\
  INTEGER1 -ne INTEGER2   INTEGER1 is not equal to INTEGER2\n", 209},
  {"\
\n\
  FILE1 -ef FILE2   FILE1 and FILE2 have the same device and inode numbers\n\
  FILE1 -nt FILE2   FILE1 is newer (modification date) than FILE2\n\
  FILE1 -ot FILE2   FILE1 is older than FILE2\n\
\n\
  -b FILE     FILE exists and is block special\n\
  -c FILE     FILE exists and is character special\n\
  -d FILE     FILE exists and is a directory\n\
  -e FILE     FILE exists\n\
  -f FILE     FILE exists and is a regular file\n\
  -g FILE     FILE exists and is set-group-ID\n\
  -G FILE     FILE exists and is owned by the effective group ID\n\
  -k FILE     FILE exists and has its sticky bit set\n\
  -L FILE     FILE exists and is a symbolic link\n\
  -O FILE     FILE exists and is owned by the effective user ID\n\
  -p FILE     FILE exists and is a named pipe\n\
  -r FILE     FILE exists and is readable\n\
  -s FILE     FILE exists and has a size greater than zero\n\
  -S FILE     FILE exists and is a socket\n\
  -t [FD]     file descriptor FD (stdout by default) is opened on a \
terminal\n\
  -u FILE     FILE exists and its set-user-ID bit is set\n\
  -w FILE     FILE exists and is writable\n\
  -x FILE     FILE exists and is executable\n", 210},
  {"\
\n\
Beware that parentheses need to be escaped (e.g., by backslashes) for \
shells.\n\
INTEGER may also be -l STRING, which evaluates to the length of STRING.\n", 211},
  {"missing `]'\n", 212},
  {"too many arguments\n", 213},
  {"\
Usage: %s [ignored command line arguments]\n\
  or:  %s OPTION\n\
Exit with a status code indicating success.\n\
\n\
These option names may not be abbreviated.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n", 214},
  {"Usage: %s [OPTION]...\n", 215},
  {"\
Print the file name of the terminal connected to standard input.\n\
\n\
  -s, --silent, --quiet   print nothing, only return an exit status\n\
      --help              display this help and exit\n\
      --version           output version information and exit\n", 216},
  {"ignoring all arguments", 217},
  {"not a tty", 218},
  {"\
Print certain system information.  With no OPTION, same as -s.\n\
\n\
  -a, --all        print all information\n\
  -m, --machine    print the machine (hardware) type\n\
  -n, --nodename   print the machine's network node hostname\n\
  -r, --release    print the operating system release\n\
  -s, --sysname    print the operating system name\n\
  -p, --processor  print the host processor type\n\
  -v               print the operating system version\n\
      --help       display this help and exit\n\
      --version    output version information and exit\n", 219},
  {"cannot get system name", 220},
  {"cannot get processor type", 221},
  {"couldn't get boot time", 222},
  {" %2d:%02d%s  up ", 223},
  {"am", 224},
  {"pm", 225},
  {"day", 226},
  {"days", 227},
  {"user", 228},
  {"users", 229},
  {",  load average: %.2f", 230},
  {"Usage: %s [OPTION]... [ FILE ]\n", 231},
  {"\
Print the current time, the length of time the system has been up,\n\
the number of users on the system, and the average number of jobs\n\
in the run queue over the last 1, 5 and 15 minutes.\n\
If FILE is not specified, use %s.  %s as FILE is common.\n\
\n\
      --help        display this help and exit\n\
      --version     output version information and exit\n", 232},
  {"\
Output who is currently logged in according to FILE.\n\
If FILE is not specified, use %s.  %s as FILE is common.\n\
\n\
      --help        display this help and exit\n\
      --version     output version information and exit\n", 233},
  {" old ", 234},
  {"\
\n\
# users=%u\n", 235},
  {"USER", 236},
  {"MESG ", 237},
  {"LINE", 238},
  {"LOGIN-TIME   ", 239},
  {"IDLE  ", 240},
  {"FROM\n", 241},
  {"Usage: %s [OPTION]... [ FILE | ARG1 ARG2 ]\n", 242},
  {"\
\n\
  -H, --heading     print line of column headings\n\
  -i, -u, --idle    add user idle time as HOURS:MINUTES, . or old\n\
  -l, --lookup      attempt to canonicalize hostnames via DNS\n\
  -m                only hostname and user associated with stdin\n\
  -q, --count       all login names and number of users logged on\n\
  -s                (ignored)\n\
  -T, -w, --mesg    add user's message status as +, - or ?\n\
      --message     same as -T\n\
      --writable    same as -T\n\
      --help        display this help and exit\n\
      --version     output version information and exit\n\
\n\
If FILE is not specified, use %s.  %s as FILE is common.\n\
If ARG1 ARG2 given, -m presumed: `am i' or `mom likes' are usual.\n", 243},
  {"\
Print the user name associated with the current effective user id.\n\
Same as id -un.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n", 244},
  {"%s: cannot find username for UID %u\n", 245},
  {"\
Usage: %s [STRING]...\n\
  or:  %s OPTION\n", 246},
  {"\
Repeatedly output a line with all specified STRING(s), or `y'.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n", 247},
};

int _msg_tbl_length = 247;
